@@ -17,9 +17,15 @@ Mostly, you'll use the renderers:
  - B{mapping} - publish a dictionary by filling slots
 """
 
+from time import time as now
 from cStringIO import StringIO
+import random
+import warnings
+
 from zope.interface import implements, providedBy
+
 import twisted.python.components as tpc
+from twisted.python.reflect import qual, accumulateClassList
 
 from nevow.context import WovenContext, NodeNotFound, PageContext
 from nevow import inevow, tags, flat, util, url
@@ -28,10 +34,24 @@ from nevow.util import log
 import formless
 from formless import iformless, annotate
 
-from time import time as now
-import random
 
-import warnings
+def _getPreprocessors(inst):
+    """
+    Accumulate elements from the sequences bound at the C{preprocessors}
+    attribute on all classes in the inheritence hierarchy of the class of
+    C{inst}.  A C{preprocessors} attribute on the given instance overrides
+    all preprocessors from the class inheritance hierarchy.
+    """
+    if 'preprocessors' in vars(inst):
+        return inst.preprocessors
+    preprocessors = []
+    accumulateClassList(
+        inst.__class__,
+        'preprocessors',
+        preprocessors)
+    return preprocessors
+
+
 
 class RenderFactory(object):
     implements(inevow.IRendererFactory)
@@ -48,6 +68,11 @@ class RenderFactory(object):
 
         callable = getattr(self, 'render_%s' % name, None)
         if callable is None:
+            warnings.warn(
+                "Renderer %r missing on %s will result in an exception." % (
+                    name, qual(type(self))),
+                category=DeprecationWarning,
+                stacklevel=1)
             callable = lambda *a, **kw: context.tag[
                 "The renderer named '%s' was not found in %r." % (name, self)]
 
@@ -371,6 +396,8 @@ class Fragment(DataFactory, RenderFactory, MacroFactory, ConfigurableMixin):
         # Remember me as lots of things
         self.rememberStuff(context)
 
+        preprocessors = _getPreprocessors(self)
+
         # This tidbit is to enable us to include Page objects inside
         # stan expressions and render_* methods and the like. But
         # because of the way objects can get intertwined, we shouldn't
@@ -380,7 +407,7 @@ class Fragment(DataFactory, RenderFactory, MacroFactory, ConfigurableMixin):
         self.docFactory.precompiledDoc = None
         try:
             try:
-                doc = self.docFactory.load(context)
+                doc = self.docFactory.load(context, preprocessors)
             finally:
                 self.docFactory.pattern = old
                 self.docFactory.precompiledDoc = None
@@ -391,12 +418,11 @@ class Fragment(DataFactory, RenderFactory, MacroFactory, ConfigurableMixin):
             # matter until it's all removed. ;-).
             if 'nevow.inevow.IQ' not in str(e):
                 raise
-            doc = self.docFactory.load(context)
+            doc = self.docFactory.load(context, preprocessors)
         except NodeNotFound:
-            doc = self.docFactory.load(context)
+            doc = self.docFactory.load(context, preprocessors)
         else:
             if old == 'content':
-                import warnings
                 warnings.warn(
                     """[v0.5] Using a Page with a 'content' pattern is
                                deprecated.""",
@@ -539,7 +565,8 @@ class Page(Fragment, ConfigurableFactory, ChildLookupMixin):
             def finisher(result):
                 return util.maybeDeferred(finishRequest).addCallback(lambda r: result)
 
-        doc = self.docFactory.load(ctx)
+        preprocessors = _getPreprocessors(self)
+        doc = self.docFactory.load(ctx, preprocessors)
         ctx =  WovenContext(ctx, tags.invisible[doc])
 
         return self.flattenFactory(doc, ctx, writer, finisher)
