@@ -260,8 +260,12 @@ class TestComplexSerialization(Base):
         ]
         self.assertEquals(self.render(tag), "<html><body><table><tr><td>Header one.</td><td>Header two.</td></tr><tr><td>One: 1</td><td>Two: 2</td></tr></table></body></html>")
 
-    def test_slotAttributeEscapingWhenPrecompiled(self):
 
+    def test_slotAttributeEscapingWhenPrecompiled(self):
+        """
+        Test that slots which represent attribute values properly quote those
+        values for that context.
+        """
         def render_searchResults(ctx, remoteCursor):
             ctx.fillSlots('old-query', '"meow"')
             return ctx.tag
@@ -275,7 +279,6 @@ class TestComplexSerialization(Base):
 
         self.assertEquals(self.render(precompiled), '<input value="&quot;meow&quot;" />')
 
-    test_slotAttributeEscapingWhenPrecompiled.todo = 'this is a bug and should be fixed.'
 
     def test_nestedpatterns(self):
         def data_table(context, data):  return [[1,2,3],[4,5,6]]
