@@ -5,6 +5,8 @@ from zope.interface import implements, Interface
 
 from twisted.internet import defer
 from twisted.trial import unittest
+from twisted.trial.util import suppress as SUPPRESS
+from twisted.python.reflect import qual
 
 from nevow import appserver
 from nevow import inevow
@@ -25,39 +27,22 @@ from formless import annotate
 from formless import iformless
 
 
-class req(testutil.FakeRequest):
-    implements(iformless.IFormDefaults)
-    method = 'GET'
-    def __init__(self):
-        testutil.FakeRequest.__init__(self, uri='/', currentSegments=[''])
-        self.d = defer.Deferred()
-        self.accumulator = ''
+def deferredRender(res, request=None):
+    if request is None:
+        request = testutil.AccumulatingFakeRequest()
 
-    def write(self, data):
-        testutil.FakeRequest.write(self, data)
-        self.accumulator+=data
-
-    def getDefault(self, key, context):
-        return ''
-
-    def remember(self, object, interface):
-        pass
-
-
-def deferredRender(res):
-    defres = req()
     d = util.maybeDeferred(res.renderHTTP,
         context.PageContext(
             tag=res, parent=context.RequestContext(
-                tag=defres)))
+                tag=request)))
 
     def done(result):
         if isinstance(result, str):
-            defres.write(result)
-        defres.d.callback(defres.accumulator)
+            request.write(result)
+        request.d.callback(request.accumulator)
         return result
     d.addCallback(done)
-    return defres.d
+    return request.d
 
 
 class TestPage(unittest.TestCase):
@@ -67,6 +52,12 @@ class TestPage(unittest.TestCase):
         r = rend.Page(docFactory=loaders.htmlstr(xhtml))
         return deferredRender(r).addCallback(
             lambda result: self.assertEquals(result, xhtml))
+    test_simple.suppress = [
+        SUPPRESS(message=
+                 r"\[v0.8\] htmlstr is deprecated because it's buggy. "
+                 "Please start using xmlfile and/or xmlstr.")]
+
+
 
     def test_extend(self):
         xhtml = '<ul id="nav"><li>one</li><li>two</li><li>three</li></ul>'
@@ -75,10 +66,17 @@ class TestPage(unittest.TestCase):
         r = R()
         return deferredRender(r).addCallback(
             lambda result: self.assertEquals(result, xhtml))
+    test_extend.suppress = [
+        SUPPRESS(message=
+                 r"\[v0.8\] htmlstr is deprecated because it's buggy. "
+                 "Please start using xmlfile and/or xmlstr.")]
+
 
     def test_data(self):
-
-        xhtml = '<ul id="nav" nevow:data="numbers" nevow:render="sequence"><li nevow:pattern="item" nevow:render="string">number</li></ul>'
+        xhtml = (
+            '<ul id="nav" nevow:data="numbers" nevow:render="sequence">'
+            '<li nevow:pattern="item" nevow:render="string">number</li>'
+            '</ul>')
 
         class R(rend.Page):
             docFactory = loaders.htmlstr(xhtml)
@@ -89,6 +87,11 @@ class TestPage(unittest.TestCase):
             lambda result: self.assertEquals(
             result,
             '<ul id="nav"><li>one</li><li>two</li><li>three</li></ul>'))
+    test_data.suppress = [
+        SUPPRESS(message=
+                 r"\[v0.8\] htmlstr is deprecated because it's buggy. "
+                 "Please start using xmlfile and/or xmlstr.")]
+
 
     def test_noData(self):
         """Test when data is missing, i.e. self.original is None and no data
@@ -100,9 +103,13 @@ class TestPage(unittest.TestCase):
         r = R()
         return deferredRender(r).addCallback(
             lambda result: self.assertIn('None', result))
+    test_noData.suppress = [
+        SUPPRESS(message=
+                 r"\[v0.8\] htmlstr is deprecated because it's buggy. "
+                 "Please start using xmlfile and/or xmlstr.")]
+
 
     def test_render(self):
-
         xhtml = '<span nevow:render="replace">replace this</span>'
 
         class R(rend.Page):
@@ -113,9 +120,13 @@ class TestPage(unittest.TestCase):
         r = R()
         return deferredRender(r).addCallback(
             lambda result: self.assertEquals(result, '<span>abc</span>'))
+    test_render.suppress = [
+        SUPPRESS(message=
+                 r"\[v0.8\] htmlstr is deprecated because it's buggy. "
+                 "Please start using xmlfile and/or xmlstr.")]
+
 
     def test_dataAndRender(self):
-
         xhtml = '''
         <table nevow:data="numbers" nevow:render="sequence">
         <tr nevow:pattern="header"><th>English</th><th>French</th></tr>
@@ -137,11 +148,23 @@ class TestPage(unittest.TestCase):
                 return context.tag
 
         r = R()
-        return deferredRender(r).addCallback(
-            lambda result: self.assertEquals(result, '<table><tr><th>English</th><th>French</th></tr><tr><td>one</td><td>un/une</td></tr><tr><td>two</td><td>deux</td></tr><tr><td>three</td><td>trois</td></tr></table>'))
+        d = deferredRender(r)
+        d.addCallback(
+            lambda result: self.assertEquals(
+                result,
+                '<table>'
+                '<tr><th>English</th><th>French</th></tr>'
+                '<tr><td>one</td><td>un/une</td></tr>'
+                '<tr><td>two</td><td>deux</td></tr>'
+                '<tr><td>three</td><td>trois</td></tr>'
+                '</table>'))
+    test_dataAndRender.suppress = [
+        SUPPRESS(message=
+                 r"\[v0.8\] htmlstr is deprecated because it's buggy. "
+                 "Please start using xmlfile and/or xmlstr.")]
+
 
     def test_stanData(self):
-
         class R(rend.Page):
             def data_numbers(context, data):
                 return ['one', 'two', 'three']
@@ -210,9 +233,9 @@ class TestPage(unittest.TestCase):
         return deferredRender(r).addCallback(
             lambda result: self.assertEquals(result, '<div id="outer"><div id="inner"></div></div>'))
 
-    def test_docFactoryInStanTree(self):
-
+    def _testDocFactoryInStanTree(self, docFactory, expected):
         class Page(rend.Page):
+            docFactory = loaders.stan(div[invisible(render=directive('included'))])
 
             def __init__(self, included):
                 self.included = included
@@ -221,34 +244,107 @@ class TestPage(unittest.TestCase):
             def render_included(self, context, data):
                 return self.included
 
-            docFactory = loaders.stan(div[invisible(render=directive('included'))])
+        return deferredRender(Page(docFactory)).addCallback(
+            self.assertEqual, '<div>' + expected + '</div>')
 
+
+    def test_stanInStanTree(self):
+        """
+        """
+        return self._testDocFactoryInStanTree(
+            loaders.stan(p['fee']),
+            '<p>fee</p>')
+
+
+    def test_htmlStringInStanTree(self):
+        """
+        Test that an htmlstr loader in a stan document is flattened by
+        having its document loaded and flattened.
+        """
+        return self._testDocFactoryInStanTree(
+            loaders.htmlstr('<p>fi</p>'),
+            '<p>fi</p>')
+    test_htmlStringInStanTree.suppress = [
+        SUPPRESS(message=
+                 r"\[v0.8\] htmlstr is deprecated because it's buggy. "
+                 "Please start using xmlfile and/or xmlstr.")]
+
+
+    def test_xmlStringInStanTree(self):
+        """
+        Like L{test_htmlStringInStanTree}, but for an xmlstr loader.
+        """
+        return self._testDocFactoryInStanTree(
+            loaders.xmlstr('<p>fo</p>'),
+            '<p>fo</p>')
+
+
+    def test_htmlFileInStanTree(self):
+        """
+        Like L{test_htmlStringInStanTree}, but for an htmlfile loader.
+        """
         doc = '<p>fum</p>'
         temp = self.mktemp()
         f = file(temp, 'w')
         f.write(doc)
         f.close()
 
-        return defer.DeferredList([
-            deferredRender(Page(loaders.stan(p['fee']))).addCallback(
-            lambda result: self.assertEquals(result, '<div><p>fee</p></div>')),
+        return self._testDocFactoryInStanTree(
+            loaders.htmlfile(temp),
+            '<p>fum</p>')
+    test_htmlFileInStanTree.suppress = [
+        SUPPRESS(message=
+                 r"\[v0.8\] htmlfile is deprecated because it's buggy. "
+                 "Please start using xmlfile and/or xmlstr.")]
 
-            deferredRender(Page(loaders.htmlstr('<p>fi</p>'))).addCallback(
-            lambda result: self.assertEquals(result, '<div><p>fi</p></div>')),
 
-            deferredRender(Page(loaders.xmlstr('<p>fo</p>'))).addCallback(
-            lambda result: self.assertEquals(result, '<div><p>fo</p></div>')),
+    def test_xmlFileInStanTree(self):
+        """
+        Like L{test_htmlStringInStanTree}, but for an xmlfile loader.
+        """
+        doc = '<p>I</p>'
+        temp = self.mktemp()
+        f = file(temp, 'w')
+        f.write(doc)
+        f.close()
 
-            deferredRender(Page(loaders.htmlfile(temp))).addCallback(
-            lambda result: self.assertEquals(result, '<div><p>fum</p></div>')),
+        return self._testDocFactoryInStanTree(
+            loaders.xmlfile(temp),
+            '<p>I</p>')
 
-            deferredRender(Page(loaders.xmlfile(temp))).addCallback(
-            lambda result: self.assertEquals(result, '<div><p>fum</p></div>'))],
-                                  fireOnOneErrback=True)
+
+    def test_reusedDocFactory(self):
+        """
+        Test that a docFactory which is used more than once behaves properly
+        both times.
+        """
+        class Page(rend.Page):
+            docFactory = loaders.stan(div[invisible(render=directive('included'))])
+
+            def __init__(self, included):
+                self.included = included
+                rend.Page.__init__(self)
+
+            def render_included(self, context, data):
+                return self.included
+
+        p1 = Page(loaders.stan('first'))
+        p2 = Page(loaders.xmlstr('<p>second</p>'))
+
+        d = deferredRender(p1)
+        def rendered(result):
+            self.assertEqual(result, '<div>first</div>')
+            return deferredRender(p2)
+        d.addCallback(rendered)
+
+        def renderedAgain(result):
+            self.assertEqual(result, '<div><p>second</p></div>')
+        d.addCallback(renderedAgain)
+
+        return d
 
 
     def test_buffered(self):
-
         class Page(rend.Page):
             buffered = True
             docFactory = loaders.stan(html[head[title['test']]])
@@ -321,16 +417,18 @@ class TestPage(unittest.TestCase):
         self.failIf("'foo' was not found" in result)
         self.failIf("'after' was not found" in result)
 
-    # Current behaviour: missing renderers etc. cause a textual
-    # warning in the web page.
-    #
-    # Wanted behaviour: missing renderers etc. raise an exception,
-    # with some helpful hints on where to locate the typo.
-    #
-    # Feel free to replace test_renderer_not_found{,_parametrized}
-    # with unit tests for the exception raising behaviour, if you do
-    # implement it.
-    def test_renderer_not_found(self):
+
+    def test_rendererNotFound(self):
+        """
+        An unparameterized renderer which is not defined should render a
+        message about the renderer not being defined.
+
+        Wanted behaviour: missing renderers etc. raise an exception, with some
+        helpful hints on where to locate the typo.
+
+        Feel free to replace test_rendererNotFound{,Parametrized} with unit
+        tests for the exception raising behaviour, if you do implement it.
+        """
         class Page(rend.Page):
             docFactory = loaders.stan(html(render=directive("notfound")))
         page = Page()
@@ -339,8 +437,17 @@ class TestPage(unittest.TestCase):
             result,
             "<html>The renderer named 'notfound' was not found in %s.</html>"
             % util.escapeToXML(repr(page)))
+    test_rendererNotFound.suppress = [
+        SUPPRESS(category=DeprecationWarning,
+            message=("Renderer 'notfound' missing on nevow.test.test_rend.Page"
+                     " will result in an exception."))]
 
-    def test_renderer_not_found_parametrized(self):
+
+    def test_rendererNotFoundParameterized(self):
+        """
+        A parameterized renderer which is not defined should render a message
+        about the renderer not being defined.
+        """
         class Page(rend.Page):
             docFactory = loaders.stan(html(render=directive("notfound dummy")))
         page = Page()
@@ -349,6 +456,29 @@ class TestPage(unittest.TestCase):
             result,
             "<html>The renderer named 'notfound' was not found in %s.</html>"
             % util.escapeToXML(repr(page)))
+    test_rendererNotFoundParameterized.suppress = [
+        SUPPRESS(
+            category=DeprecationWarning,
+            message=("Renderer 'notfound' missing on nevow.test.test_rend.Page"
+                     " will result in an exception."))]
+
+
+    def test_missingRendererDeprecated(self):
+        """
+        Using a renderer which is not defined should emit a deprecation
+        warning.
+        """
+        renderer = "notfound"
+        class MisdefinedPage(rend.Page):
+            docFactory = loaders.stan(html(render=directive(renderer)))
+        page = MisdefinedPage()
+        message = "Renderer %r missing on %s will result in an exception."
+        message %= (renderer, qual(MisdefinedPage))
+        self.assertWarns(
+            DeprecationWarning, message, rend.__file__,
+            page.renderSynchronously)
+
+
 
 class TestFragment(unittest.TestCase):
 
@@ -868,3 +998,4 @@ class TestMacro(unittest.TestCase):
         p2_str = p2.renderSynchronously(ctx2)
 
         self.assertEquals(p1_str, p2_str)
+
